@@ -80,7 +80,7 @@ const RecipeHelpers = {
 
 // Initialization functions common to all tests
 
-add_task(async function test_common_initialize() {
+add_setup(async function test_common_initialize() {
   // Before initializing the service for the first time, we should copy the key
   // file required to decrypt the logins contained in the SQLite databases used
   // by migration tests.  This file is not required for the other tests.
@@ -100,7 +100,7 @@ add_task(async function test_common_initialize() {
   }
 });
 
-add_task(async function test_common_prefs() {
+add_setup(async function test_common_prefs() {
   Services.prefs.setStringPref(NEW_PASSWORD_HEURISTIC_ENABLED_PREF, "0.75");
 });
 
