@@ -12,6 +12,11 @@ const Cm = Components.manager;
 const { XPCOMUtils } = ChromeUtils.import(
   "resource://gre/modules/XPCOMUtils.jsm"
 );
+ChromeUtils.defineModuleGetter(
+  this,
+  "Services",
+  "resource://gre/modules/Services.jsm"
+);
 
 const CATEGORY_UPDATE_TIMER = "update-timer";
 
@@ -137,20 +142,6 @@ const TESTS = [
 var gUTM;
 var gNextFunc;
 
-XPCOMUtils.defineLazyServiceGetter(
-  this,
-  "gPref",
-  "@mozilla.org/preferences-service;1",
-  "nsIPrefBranch"
-);
-
-XPCOMUtils.defineLazyServiceGetter(
-  this,
-  "gCatMan",
-  "@mozilla.org/categorymanager;1",
-  "nsICategoryManager"
-);
-
 XPCOMUtils.defineLazyGetter(this, "gCompReg", function() {
   return Cm.QueryInterface(Ci.nsIComponentRegistrar);
 });
@@ -161,15 +152,12 @@ const gTest0TimerCallback = {
     // time to fire so check other timers are successful.
     do_throw("gTest0TimerCallback notify method should not have been called");
   },
-  QueryInterface: ChromeUtils.generateQI([Ci.nsITimerCallback]),
+  QueryInterface: ChromeUtils.generateQI(["nsITimerCallback"]),
 };
 
 const gTest0Factory = {
-  createInstance: function T0F_createInstance(aOuter, aIID) {
-    if (aOuter == null) {
-      return gTest0TimerCallback.QueryInterface(aIID);
-    }
-    throw Cr.NS_ERROR_NO_AGGREGATION;
+  createInstance: function T0F_createInstance(aIID) {
+    return gTest0TimerCallback.QueryInterface(aIID);
   },
 };
 
@@ -179,15 +167,12 @@ const gTest1TimerCallback = {
     // time to fire so check other timers are successful.
     do_throw("gTest1TimerCallback notify method should not have been called");
   },
-  QueryInterface: ChromeUtils.generateQI([Ci.nsITimer]),
+  QueryInterface: ChromeUtils.generateQI(["nsITimer"]),
 };
 
 const gTest1Factory = {
-  createInstance: function T1F_createInstance(aOuter, aIID) {
-    if (aOuter == null) {
-      return gTest1TimerCallback.QueryInterface(aIID);
-    }
-    throw Cr.NS_ERROR_NO_AGGREGATION;
+  createInstance: function T1F_createInstance(aIID) {
+    return gTest1TimerCallback.QueryInterface(aIID);
   },
 };
 
@@ -197,100 +182,98 @@ const gTest2TimerCallback = {
     // time to fire so check other timers are successful.
     do_throw("gTest2TimerCallback notify method should not have been called");
   },
-  QueryInterface: ChromeUtils.generateQI([Ci.nsITimerCallback]),
+  QueryInterface: ChromeUtils.generateQI(["nsITimerCallback"]),
 };
 
 const gTest2Factory = {
-  createInstance: function T2F_createInstance(aOuter, aIID) {
-    if (aOuter == null) {
-      return gTest2TimerCallback.QueryInterface(aIID);
-    }
-    throw Cr.NS_ERROR_NO_AGGREGATION;
+  createInstance: function T2F_createInstance(aIID) {
+    return gTest2TimerCallback.QueryInterface(aIID);
   },
 };
 
 const gTest3TimerCallback = {
-  QueryInterface: ChromeUtils.generateQI([Ci.nsITimerCallback]),
+  QueryInterface: ChromeUtils.generateQI(["nsITimerCallback"]),
 };
 
 const gTest3Factory = {
-  createInstance: function T3F_createInstance(aOuter, aIID) {
-    if (aOuter == null) {
-      return gTest3TimerCallback.QueryInterface(aIID);
-    }
-    throw Cr.NS_ERROR_NO_AGGREGATION;
+  createInstance: function T3F_createInstance(aIID) {
+    return gTest3TimerCallback.QueryInterface(aIID);
   },
 };
 
 const gTest4TimerCallback = {
   notify: function T4CB_notify(aTimer) {
-    gCatMan.deleteCategoryEntry(CATEGORY_UPDATE_TIMER, TESTS[4].desc, true);
+    Services.catMan.deleteCategoryEntry(
+      CATEGORY_UPDATE_TIMER,
+      TESTS[4].desc,
+      true
+    );
     TESTS[4].notified = true;
     finished_test0thru7();
   },
-  QueryInterface: ChromeUtils.generateQI([Ci.nsITimerCallback]),
+  QueryInterface: ChromeUtils.generateQI(["nsITimerCallback"]),
 };
 
 const gTest4Factory = {
-  createInstance: function T4F_createInstance(aOuter, aIID) {
-    if (aOuter == null) {
-      return gTest4TimerCallback.QueryInterface(aIID);
-    }
-    throw Cr.NS_ERROR_NO_AGGREGATION;
+  createInstance: function T4F_createInstance(aIID) {
+    return gTest4TimerCallback.QueryInterface(aIID);
   },
 };
 
 const gTest5TimerCallback = {
   notify: function T5CB_notify(aTimer) {
-    gCatMan.deleteCategoryEntry(CATEGORY_UPDATE_TIMER, TESTS[5].desc, true);
+    Services.catMan.deleteCategoryEntry(
+      CATEGORY_UPDATE_TIMER,
+      TESTS[5].desc,
+      true
+    );
     TESTS[5].notified = true;
     finished_test0thru7();
   },
-  QueryInterface: ChromeUtils.generateQI([Ci.nsITimerCallback]),
+  QueryInterface: ChromeUtils.generateQI(["nsITimerCallback"]),
 };
 
 const gTest5Factory = {
-  createInstance: function T5F_createInstance(aOuter, aIID) {
-    if (aOuter == null) {
-      return gTest5TimerCallback.QueryInterface(aIID);
-    }
-    throw Cr.NS_ERROR_NO_AGGREGATION;
+  createInstance: function T5F_createInstance(aIID) {
+    return gTest5TimerCallback.QueryInterface(aIID);
   },
 };
 
 const gTest6TimerCallback = {
   notify: function T6CB_notify(aTimer) {
-    gCatMan.deleteCategoryEntry(CATEGORY_UPDATE_TIMER, TESTS[6].desc, true);
+    Services.catMan.deleteCategoryEntry(
+      CATEGORY_UPDATE_TIMER,
+      TESTS[6].desc,
+      true
+    );
     TESTS[6].notified = true;
     finished_test0thru7();
   },
-  QueryInterface: ChromeUtils.generateQI([Ci.nsITimerCallback]),
+  QueryInterface: ChromeUtils.generateQI(["nsITimerCallback"]),
 };
 
 const gTest6Factory = {
-  createInstance: function T6F_createInstance(aOuter, aIID) {
-    if (aOuter == null) {
-      return gTest6TimerCallback.QueryInterface(aIID);
-    }
-    throw Cr.NS_ERROR_NO_AGGREGATION;
+  createInstance: function T6F_createInstance(aIID) {
+    return gTest6TimerCallback.QueryInterface(aIID);
   },
 };
 
 const gTest7TimerCallback = {
   notify: function T7CB_notify(aTimer) {
-    gCatMan.deleteCategoryEntry(CATEGORY_UPDATE_TIMER, TESTS[7].desc, true);
+    Services.catMan.deleteCategoryEntry(
+      CATEGORY_UPDATE_TIMER,
+      TESTS[7].desc,
+      true
+    );
     TESTS[7].notified = true;
     finished_test0thru7();
   },
-  QueryInterface: ChromeUtils.generateQI([Ci.nsITimerCallback]),
+  QueryInterface: ChromeUtils.generateQI(["nsITimerCallback"]),
 };
 
 const gTest7Factory = {
-  createInstance: function T7F_createInstance(aOuter, aIID) {
-    if (aOuter == null) {
-      return gTest7TimerCallback.QueryInterface(aIID);
-    }
-    throw Cr.NS_ERROR_NO_AGGREGATION;
+  createInstance: function T7F_createInstance(aIID) {
+    return gTest7TimerCallback.QueryInterface(aIID);
   },
 };
 
@@ -302,15 +285,12 @@ const gTest8TimerCallback = {
       check_test8thru10(gTest8TimerCallback);
     });
   },
-  QueryInterface: ChromeUtils.generateQI([Ci.nsITimerCallback]),
+  QueryInterface: ChromeUtils.generateQI(["nsITimerCallback"]),
 };
 
 const gTest8Factory = {
-  createInstance: function T8F_createInstance(aOuter, aIID) {
-    if (aOuter == null) {
-      return gTest8TimerCallback.QueryInterface(aIID);
-    }
-    throw Cr.NS_ERROR_NO_AGGREGATION;
+  createInstance: function T8F_createInstance(aIID) {
+    return gTest8TimerCallback.QueryInterface(aIID);
   },
 };
 
@@ -322,7 +302,7 @@ const gTest9TimerCallback = {
       check_test8thru10(gTest9TimerCallback);
     });
   },
-  QueryInterface: ChromeUtils.generateQI([Ci.nsITimerCallback]),
+  QueryInterface: ChromeUtils.generateQI(["nsITimerCallback"]),
 };
 
 const gTest10TimerCallback = {
@@ -331,15 +311,12 @@ const gTest10TimerCallback = {
     // be called.
     do_throw("gTest10TimerCallback notify method should not have been called");
   },
-  QueryInterface: ChromeUtils.generateQI([Ci.nsITimerCallback]),
+  QueryInterface: ChromeUtils.generateQI(["nsITimerCallback"]),
 };
 
 const gTest9Factory = {
-  createInstance: function T9F_createInstance(aOuter, aIID) {
-    if (aOuter == null) {
-      return gTest9TimerCallback.QueryInterface(aIID);
-    }
-    throw Cr.NS_ERROR_NO_AGGREGATION;
+  createInstance: function T9F_createInstance(aIID) {
+    return gTest9TimerCallback.QueryInterface(aIID);
   },
 };
 
@@ -347,18 +324,21 @@ function run_test() {
   do_test_pending();
 
   // Set the timer to fire every second
-  gPref.setIntPref(
+  Services.prefs.setIntPref(
     PREF_APP_UPDATE_TIMERMINIMUMDELAY,
     MAIN_TIMER_INTERVAL / 1000
   );
-  gPref.setIntPref(PREF_APP_UPDATE_TIMERFIRSTINTERVAL, MAIN_TIMER_INTERVAL);
-  gPref.setBoolPref(PREF_APP_UPDATE_LOG_ALL, true);
+  Services.prefs.setIntPref(
+    PREF_APP_UPDATE_TIMERFIRSTINTERVAL,
+    MAIN_TIMER_INTERVAL
+  );
+  Services.prefs.setBoolPref(PREF_APP_UPDATE_LOG_ALL, true);
 
   // Remove existing update timers to prevent them from being notified
-  for (let { data: entry } of gCatMan.enumerateCategory(
+  for (let { data: entry } of Services.catMan.enumerateCategory(
     CATEGORY_UPDATE_TIMER
   )) {
-    gCatMan.deleteCategoryEntry(CATEGORY_UPDATE_TIMER, entry, false);
+    Services.catMan.deleteCategoryEntry(CATEGORY_UPDATE_TIMER, entry, false);
   }
 
   gUTM = Cc["@mozilla.org/updates/timer-manager;1"]
@@ -383,7 +363,7 @@ function run_test0thru7() {
     TESTS[0].contractID,
     gTest0Factory
   );
-  gCatMan.addCategoryEntry(
+  Services.catMan.addCategoryEntry(
     CATEGORY_UPDATE_TIMER,
     TESTS[0].desc,
     [
@@ -404,7 +384,7 @@ function run_test0thru7() {
     TESTS[1].contractID,
     gTest1Factory
   );
-  gCatMan.addCategoryEntry(
+  Services.catMan.addCategoryEntry(
     CATEGORY_UPDATE_TIMER,
     TESTS[1].desc,
     [
@@ -420,7 +400,7 @@ function run_test0thru7() {
 
   // has a last update time of now - 43200 which is half of its interval
   let lastUpdateTime = Math.round(Date.now() / 1000) - 43200;
-  gPref.setIntPref(
+  Services.prefs.setIntPref(
     PREF_BRANCH_LAST_UPDATE_TIME + TESTS[2].timerID,
     lastUpdateTime
   );
@@ -430,7 +410,7 @@ function run_test0thru7() {
     TESTS[2].contractID,
     gTest2Factory
   );
-  gCatMan.addCategoryEntry(
+  Services.catMan.addCategoryEntry(
     CATEGORY_UPDATE_TIMER,
     TESTS[2].desc,
     [
@@ -451,7 +431,7 @@ function run_test0thru7() {
     TESTS[3].contractID,
     gTest3Factory
   );
-  gCatMan.addCategoryEntry(
+  Services.catMan.addCategoryEntry(
     CATEGORY_UPDATE_TIMER,
     TESTS[3].desc,
     [
@@ -466,14 +446,14 @@ function run_test0thru7() {
   );
 
   // already has a last update time
-  gPref.setIntPref(PREF_BRANCH_LAST_UPDATE_TIME + TESTS[4].timerID, 1);
+  Services.prefs.setIntPref(PREF_BRANCH_LAST_UPDATE_TIME + TESTS[4].timerID, 1);
   gCompReg.registerFactory(
     TESTS[4].classID,
     TESTS[4].desc,
     TESTS[4].contractID,
     gTest4Factory
   );
-  gCatMan.addCategoryEntry(
+  Services.catMan.addCategoryEntry(
     CATEGORY_UPDATE_TIMER,
     TESTS[4].desc,
     [
@@ -488,14 +468,14 @@ function run_test0thru7() {
   );
 
   // has an interval preference that overrides the default
-  gPref.setIntPref(TESTS[5].prefInterval, CONSUMER_TIMER_INTERVAL);
+  Services.prefs.setIntPref(TESTS[5].prefInterval, CONSUMER_TIMER_INTERVAL);
   gCompReg.registerFactory(
     TESTS[5].classID,
     TESTS[5].desc,
     TESTS[5].contractID,
     gTest5Factory
   );
-  gCatMan.addCategoryEntry(
+  Services.catMan.addCategoryEntry(
     CATEGORY_UPDATE_TIMER,
     TESTS[5].desc,
     [
@@ -511,7 +491,7 @@ function run_test0thru7() {
 
   // has a next update time 24 hours from now
   let nextUpdateTime = Math.round(Date.now() / 1000) + 86400;
-  gPref.setIntPref(
+  Services.prefs.setIntPref(
     PREF_BRANCH_LAST_UPDATE_TIME + TESTS[6].timerID,
     nextUpdateTime
   );
@@ -521,7 +501,7 @@ function run_test0thru7() {
     TESTS[6].contractID,
     gTest6Factory
   );
-  gCatMan.addCategoryEntry(
+  Services.catMan.addCategoryEntry(
     CATEGORY_UPDATE_TIMER,
     TESTS[6].desc,
     [
@@ -536,14 +516,14 @@ function run_test0thru7() {
   );
 
   // has a maximum interval set by the value of MAIN_TIMER_INTERVAL
-  gPref.setIntPref(TESTS[7].prefInterval, 86400);
+  Services.prefs.setIntPref(TESTS[7].prefInterval, 86400);
   gCompReg.registerFactory(
     TESTS[7].classID,
     TESTS[7].desc,
     TESTS[7].contractID,
     gTest7Factory
   );
-  gCatMan.addCategoryEntry(
+  Services.catMan.addCategoryEntry(
     CATEGORY_UPDATE_TIMER,
     TESTS[7].desc,
     [
@@ -610,28 +590,48 @@ function check_test0thru7() {
   );
 
   Assert.ok(
-    gPref.prefHasUserValue(PREF_BRANCH_LAST_UPDATE_TIME + TESTS[4].timerID),
+    Services.prefs.prefHasUserValue(
+      PREF_BRANCH_LAST_UPDATE_TIME + TESTS[4].timerID
+    ),
     "first of two category registered timers last update time has " +
       "a user value"
   );
   Assert.ok(
-    gPref.prefHasUserValue(PREF_BRANCH_LAST_UPDATE_TIME + TESTS[5].timerID),
+    Services.prefs.prefHasUserValue(
+      PREF_BRANCH_LAST_UPDATE_TIME + TESTS[5].timerID
+    ),
     "second of two category registered timers last update time has " +
       "a user value"
   );
 
   // Remove the category timers that should have failed
-  gCatMan.deleteCategoryEntry(CATEGORY_UPDATE_TIMER, TESTS[0].desc, true);
-  gCatMan.deleteCategoryEntry(CATEGORY_UPDATE_TIMER, TESTS[1].desc, true);
-  gCatMan.deleteCategoryEntry(CATEGORY_UPDATE_TIMER, TESTS[2].desc, true);
-  gCatMan.deleteCategoryEntry(CATEGORY_UPDATE_TIMER, TESTS[3].desc, true);
-  for (let { data: entry } of gCatMan.enumerateCategory(
+  Services.catMan.deleteCategoryEntry(
+    CATEGORY_UPDATE_TIMER,
+    TESTS[0].desc,
+    true
+  );
+  Services.catMan.deleteCategoryEntry(
+    CATEGORY_UPDATE_TIMER,
+    TESTS[1].desc,
+    true
+  );
+  Services.catMan.deleteCategoryEntry(
+    CATEGORY_UPDATE_TIMER,
+    TESTS[2].desc,
+    true
+  );
+  Services.catMan.deleteCategoryEntry(
+    CATEGORY_UPDATE_TIMER,
+    TESTS[3].desc,
+    true
+  );
+  for (let { data: entry } of Services.catMan.enumerateCategory(
     CATEGORY_UPDATE_TIMER
   )) {
-    gCatMan.deleteCategoryEntry(CATEGORY_UPDATE_TIMER, entry, false);
+    Services.catMan.deleteCategoryEntry(CATEGORY_UPDATE_TIMER, entry, false);
   }
 
-  let entries = gCatMan.enumerateCategory(CATEGORY_UPDATE_TIMER);
+  let entries = Services.catMan.enumerateCategory(CATEGORY_UPDATE_TIMER);
   Assert.ok(
     !entries.hasMoreElements(),
     "no " +
@@ -644,7 +644,7 @@ function check_test0thru7() {
 }
 
 function run_test8thru10() {
-  gPref.setIntPref(PREF_BRANCH_LAST_UPDATE_TIME + TESTS[8].timerID, 1);
+  Services.prefs.setIntPref(PREF_BRANCH_LAST_UPDATE_TIME + TESTS[8].timerID, 1);
   gCompReg.registerFactory(
     TESTS[8].classID,
     TESTS[8].desc,
@@ -656,7 +656,7 @@ function run_test8thru10() {
     gTest8TimerCallback,
     TESTS[8].defaultInterval
   );
-  gPref.setIntPref(PREF_BRANCH_LAST_UPDATE_TIME + TESTS[9].timerID, 1);
+  Services.prefs.setIntPref(PREF_BRANCH_LAST_UPDATE_TIME + TESTS[9].timerID, 1);
   gCompReg.registerFactory(
     TESTS[9].classID,
     TESTS[9].desc,
@@ -702,7 +702,9 @@ function check_test8thru10(aTestTimerCallback) {
       "time should have occured"
   );
 
-  let time = gPref.getIntPref(PREF_BRANCH_LAST_UPDATE_TIME + TESTS[8].timerID);
+  let time = Services.prefs.getIntPref(
+    PREF_BRANCH_LAST_UPDATE_TIME + TESTS[8].timerID
+  );
   Assert.notEqual(
     time,
     1,
@@ -710,7 +712,9 @@ function check_test8thru10(aTestTimerCallback) {
       "should have been updated"
   );
 
-  time = gPref.getIntPref(PREF_BRANCH_LAST_UPDATE_TIME + TESTS[9].timerID);
+  time = Services.prefs.getIntPref(
+    PREF_BRANCH_LAST_UPDATE_TIME + TESTS[9].timerID
+  );
   Assert.notEqual(
     time,
     1,
