@@ -10,12 +10,10 @@ const { TelemetryTestUtils } = ChromeUtils.import(
   "resource://testing-common/TelemetryTestUtils.jsm"
 );
 
-var tmp = {};
-ChromeUtils.import("resource://gre/modules/AddonManager.jsm", tmp);
-ChromeUtils.import("resource://gre/modules/Log.jsm", tmp);
-var AddonManager = tmp.AddonManager;
-var AddonManagerPrivate = tmp.AddonManagerPrivate;
-var Log = tmp.Log;
+let { AddonManager, AddonManagerPrivate } = ChromeUtils.import(
+  "resource://gre/modules/AddonManager.jsm"
+);
+let { Log } = ChromeUtils.import("resource://gre/modules/Log.jsm");
 
 var pathParts = gTestPath.split("/");
 // Drop the test filename
@@ -28,7 +26,6 @@ const SECURE_TESTROOT = "https://example.com/" + RELATIVE_DIR;
 const TESTROOT2 = "http://example.org/" + RELATIVE_DIR;
 const SECURE_TESTROOT2 = "https://example.org/" + RELATIVE_DIR;
 const CHROMEROOT = pathParts.join("/") + "/";
-const PREF_DISCOVERURL = "extensions.webservice.discoverURL";
 const PREF_DISCOVER_ENABLED = "extensions.getAddons.showPane";
 const PREF_XPI_ENABLED = "xpinstall.enabled";
 const PREF_UPDATEURL = "extensions.update.url";
@@ -99,29 +96,13 @@ function promiseFocus(window) {
   return new Promise(resolve => waitForFocus(resolve, window));
 }
 
-// Helper to register test failures and close windows if any are left open
-function checkOpenWindows(aWindowID) {
-  let found = false;
-  for (let win of Services.wm.getEnumerator(aWindowID)) {
-    if (!win.closed) {
-      found = true;
-      win.close();
-    }
-  }
-  if (found) {
-    ok(false, "Found unexpected " + aWindowID + " window still open");
-  }
-}
-
 // Tools to disable and re-enable the background update and blocklist timers
 // so that tests can protect themselves from unwanted timer events.
 var gCatMan = Services.catMan;
-// Default values from toolkit/mozapps/extensions/extensions.manifest, but disable*UpdateTimer()
+// Default value from toolkit/mozapps/extensions/extensions.manifest, but disable*UpdateTimer()
 // records the actual value so we can put it back in enable*UpdateTimer()
 var backgroundUpdateConfig =
   "@mozilla.org/addons/integration;1,getService,addon-background-update-timer,extensions.update.interval,86400";
-var blocklistUpdateConfig =
-  "@mozilla.org/extensions/blocklist;1,getService,blocklist-background-update-timer,extensions.blocklist.interval,86400";
 
 var UTIMER = "update-timer";
 var AMANAGER = "addonManager";
@@ -144,23 +125,6 @@ function enableBackgroundUpdateTimer() {
   );
 }
 
-function disableBlocklistUpdateTimer() {
-  info("Disabling " + UTIMER + " " + BLOCKLIST);
-  blocklistUpdateConfig = gCatMan.getCategoryEntry(UTIMER, BLOCKLIST);
-  gCatMan.deleteCategoryEntry(UTIMER, BLOCKLIST, true);
-}
-
-function enableBlocklistUpdateTimer() {
-  info("Enabling " + UTIMER + " " + BLOCKLIST);
-  gCatMan.addCategoryEntry(
-    UTIMER,
-    BLOCKLIST,
-    blocklistUpdateConfig,
-    false,
-    true
-  );
-}
-
 registerCleanupFunction(function() {
   // Restore prefs
   for (let pref of gRestorePrefs) {
@@ -175,11 +139,6 @@ registerCleanupFunction(function() {
     }
   }
 
-  // Throw an error if the add-ons manager window is open anywhere
-  checkOpenWindows("Addons:Manager");
-  checkOpenWindows("Addons:Compatibility");
-  checkOpenWindows("Addons:Install");
-
   return AddonManager.getAllInstalls().then(aInstalls => {
     for (let install of aInstalls) {
       if (install instanceof MockInstall) {
@@ -230,7 +189,7 @@ function run_next_test() {
     info("Test " + gTestsRun + " took " + (Date.now() - gTestStart) + "ms");
   }
 
-  if (gPendingTests.length == 0) {
+  if (!gPendingTests.length) {
     executeSoon(end_test);
     return;
   }
@@ -248,63 +207,22 @@ function run_next_test() {
 }
 
 var get_tooltip_info = async function(addonEl, managerWindow) {
-  if (managerWindow && managerWindow.useHtmlViews) {
-    // Extract from title attribute.
-    const { addon } = addonEl;
-    const name = addon.name;
-    const nameEl = addonEl.addonNameEl;
+  // Extract from title attribute.
+  const { addon } = addonEl;
+  const name = addon.name;
 
-    let nameWithVersion = nameEl.title;
-    if (addonEl.addon.userDisabled) {
-      // TODO - Bug 1558077: Currently Fluent is clearing the addon title
-      // when the addon is disabled, fixing it requires changes to the
-      // HTML about:addons localized strings, and then remove this
-      // workaround.
-      nameWithVersion = `${name} ${addon.version}`;
-    }
-
-    return {
-      name,
-      version: nameWithVersion.substring(name.length + 1),
-    };
+  let nameWithVersion = addonEl.addonNameEl.title;
+  if (addonEl.addon.userDisabled) {
+    // TODO - Bug 1558077: Currently Fluent is clearing the addon title
+    // when the addon is disabled, fixing it requires changes to the
+    // HTML about:addons localized strings, and then remove this
+    // workaround.
+    nameWithVersion = `${name} ${addon.version}`;
   }
 
-  // Retrieve the tooltip from the XUL about:addons view,
-  // the popup code uses a triggering event's target to set the
-  // document.tooltipNode property.
-  let doc = addonEl.ownerDocument;
-  let nameNode = doc.getAnonymousElementByAttribute(addonEl, "anonid", "name");
-  let event = new doc.ownerGlobal.CustomEvent("TriggerEvent");
-  nameNode.dispatchEvent(event);
-
-  let tooltip = doc.getElementById("addonitem-tooltip");
-
-  let promise = BrowserTestUtils.waitForEvent(tooltip, "popupshown");
-  tooltip.openPopup(nameNode, "after_start", 0, 0, false, false, event);
-  await promise;
-
-  let tiptext = tooltip.label;
-
-  promise = BrowserTestUtils.waitForEvent(tooltip, "popuphidden");
-  tooltip.hidePopup();
-  await promise;
-
-  let expectedName = addonEl.getAttribute("name");
-  is(
-    tiptext.substring(0, expectedName.length),
-    expectedName,
-    "Tooltip should always start with the expected name"
-  );
-
-  if (expectedName.length == tiptext.length) {
-    return {
-      name: tiptext,
-      version: undefined,
-    };
-  }
   return {
-    name: tiptext.substring(0, expectedName.length),
-    version: tiptext.substring(expectedName.length + 1),
+    name,
+    version: nameWithVersion.substring(name.length + 1),
   };
 };
 
@@ -326,20 +244,6 @@ function get_addon_file_url(aFilename) {
   }
 }
 
-function get_current_view(aManager) {
-  let view = aManager.document.getElementById("view-port").selectedPanel;
-  if (view.id == "headered-views") {
-    view = aManager.document.getElementById("headered-views-content")
-      .selectedPanel;
-  }
-  is(
-    view,
-    aManager.gViewController.displayedView,
-    "view controller is tracking the displayed view correctly"
-  );
-  return view;
-}
-
 function check_all_in_list(aManager, aIds, aIgnoreExtras) {
   var doc = aManager.document;
   var list = doc.getElementById("addon-list");
@@ -370,37 +274,23 @@ function check_all_in_list(aManager, aIds, aIgnoreExtras) {
   }
 }
 
-function get_addon_element(aManager, aId) {
-  if (aManager.useHtmlViews) {
-    const doc = aManager.getHtmlBrowser().contentDocument;
-    return doc.querySelector(`addon-card[addon-id="${aId}"]`);
-  }
-
-  const doc = aManager.document;
-  const view = get_current_view(aManager);
-  let listid = "addon-list";
-  if (view.id == "updates-view") {
-    listid = "updates-list";
-  }
-  const list = doc.getElementById(listid);
-
-  let node = list.firstChild;
-  while (node) {
-    if (node.value == aId) {
-      return node;
-    }
-    node = node.nextSibling;
-  }
-
-  return null;
+function getAddonCard(win, id) {
+  return win.document.querySelector(`addon-card[addon-id="${id}"]`);
 }
 
-function wait_for_view_load(
+async function wait_for_view_load(
   aManagerWindow,
   aCallback,
   aForceWait,
   aLongerTimeout
 ) {
+  // Wait one tick to make sure that the microtask related to an
+  // async loadView call originated from outsite about:addons
+  // is already executing (otherwise isLoading would be still false
+  // and we wouldn't be waiting for that load before resolving
+  // the promise returned by this test helper function).
+  await Promise.resolve();
+
   let p = new Promise(resolve => {
     requestLongerTimeout(aLongerTimeout ? aLongerTimeout : 2);
 
@@ -410,7 +300,7 @@ function wait_for_view_load(
     }
 
     aManagerWindow.document.addEventListener(
-      "ViewChanged",
+      "view-loaded",
       function() {
         resolve(aManagerWindow);
       },
@@ -422,23 +312,11 @@ function wait_for_view_load(
 }
 
 function wait_for_manager_load(aManagerWindow, aCallback) {
-  let p = new Promise(resolve => {
-    if (!aManagerWindow.gIsInitializing) {
-      resolve(aManagerWindow);
-      return;
-    }
-
-    info("Waiting for initialization");
-    aManagerWindow.document.addEventListener(
-      "Initialized",
-      function() {
-        resolve(aManagerWindow);
-      },
-      { once: true }
-    );
-  });
-
-  return log_callback(p, aCallback);
+  info("Waiting for initialization");
+  return log_callback(
+    aManagerWindow.promiseInitialized.then(() => aManagerWindow),
+    aCallback
+  );
 }
 
 function open_manager(
@@ -460,7 +338,7 @@ function open_manager(
 
       ok(aManagerWindow != null, "Should have an add-ons manager window");
       is(
-        aManagerWindow.location,
+        aManagerWindow.location.href,
         MANAGER_URI,
         "Should be displaying the correct UI"
       );
@@ -502,7 +380,7 @@ function close_manager(aManagerWindow, aCallback, aLongerTimeout) {
       "Should have an add-ons manager window to close"
     );
     is(
-      aManagerWindow.location,
+      aManagerWindow.location.href,
       MANAGER_URI,
       "Should be closing window with correct URI"
     );
@@ -560,16 +438,6 @@ function wait_for_window_open(aCallback) {
   return log_callback(p, aCallback);
 }
 
-function get_string(aName, ...aArgs) {
-  var bundle = Services.strings.createBundle(
-    "chrome://mozapps/locale/extensions/extensions.properties"
-  );
-  if (aArgs.length == 0) {
-    return bundle.GetStringFromName(aName);
-  }
-  return bundle.formatStringFromName(aName, aArgs, aArgs.length);
-}
-
 function formatDate(aDate) {
   const dtOptions = { year: "numeric", month: "long", day: "numeric" };
   return aDate.toLocaleDateString(undefined, dtOptions);
@@ -614,9 +482,9 @@ function promiseAddonsByIDs(aIDs) {
  *
  * The callback will receive the Addon for the installed add-on.
  */
-function install_addon(path, cb, pathPrefix = TESTROOT) {
-  let p = new Promise(async (resolve, reject) => {
-    let install = await AddonManager.getInstallForURL(pathPrefix + path);
+async function install_addon(path, cb, pathPrefix = TESTROOT) {
+  let install = await AddonManager.getInstallForURL(pathPrefix + path);
+  let p = new Promise((resolve, reject) => {
     install.addListener({
       onInstallEnded: () => resolve(install.addon),
     });
@@ -629,109 +497,93 @@ function install_addon(path, cb, pathPrefix = TESTROOT) {
 
 function CategoryUtilities(aManagerWindow) {
   this.window = aManagerWindow;
-
-  var self = this;
-  this.window.addEventListener(
-    "unload",
-    function() {
-      self.window = null;
-    },
-    { once: true }
-  );
+  this.window.addEventListener("unload", () => (this.window = null), {
+    once: true,
+  });
 }
 
 CategoryUtilities.prototype = {
   window: null,
 
+  get _categoriesBox() {
+    return this.window.document.querySelector("categories-box");
+  },
+
+  getSelectedViewId() {
+    let selectedItem = this._categoriesBox.querySelector("[selected]");
+    isnot(selectedItem, null, "A category should be selected");
+    return selectedItem.getAttribute("viewid");
+  },
+
   get selectedCategory() {
     isnot(
       this.window,
       null,
       "Should not get selected category when manager window is not loaded"
     );
-    var selectedItem = this.window.document.getElementById("categories")
-      .selectedItem;
-    isnot(selectedItem, null, "A category should be selected");
-    var view = this.window.gViewController.parseViewId(selectedItem.value);
+    let viewId = this.getSelectedViewId();
+    let view = this.window.gViewController.parseViewId(viewId);
     return view.type == "list" ? view.param : view.type;
   },
 
-  get(aCategoryType, aAllowMissing) {
+  get(categoryType) {
     isnot(
       this.window,
       null,
       "Should not get category when manager window is not loaded"
     );
-    var categories = this.window.document.getElementById("categories");
 
-    var viewId = "addons://list/" + aCategoryType;
-    var items = categories.getElementsByAttribute("value", viewId);
-    if (items.length) {
-      return items[0];
+    let button = this._categoriesBox.querySelector(`[name="${categoryType}"]`);
+    if (button) {
+      return button;
     }
 
-    viewId = "addons://" + aCategoryType + "/";
-    items = categories.getElementsByAttribute("value", viewId);
-    if (items.length) {
-      return items[0];
-    }
-
-    if (!aAllowMissing) {
-      ok(false, "Should have found a category with type " + aCategoryType);
-    }
+    ok(false, "Should have found a category with type " + categoryType);
     return null;
   },
 
-  getViewId(aCategoryType) {
-    isnot(
-      this.window,
-      null,
-      "Should not get view id when manager window is not loaded"
-    );
-    return this.get(aCategoryType).value;
-  },
-
-  isVisible(aCategory) {
+  isVisible(categoryButton) {
     isnot(
       this.window,
       null,
       "Should not check visible state when manager window is not loaded"
     );
-    if (
-      aCategory.hasAttribute("disabled") &&
-      aCategory.getAttribute("disabled") == "true"
-    ) {
+
+    // There are some tests checking this before the categories have loaded.
+    if (!categoryButton) {
       return false;
     }
 
-    return !is_hidden(aCategory);
+    if (categoryButton.disabled || categoryButton.hidden) {
+      return false;
+    }
+
+    return !is_hidden(categoryButton);
   },
 
-  isTypeVisible(aCategoryType) {
-    return this.isVisible(this.get(aCategoryType));
+  isTypeVisible(categoryType) {
+    return this.isVisible(this.get(categoryType));
   },
 
-  open(aCategory, aCallback) {
+  open(categoryButton) {
     isnot(
       this.window,
       null,
       "Should not open category when manager window is not loaded"
     );
     ok(
-      this.isVisible(aCategory),
+      this.isVisible(categoryButton),
       "Category should be visible if attempting to open it"
     );
 
-    EventUtils.synthesizeMouse(aCategory, 2, 2, {}, this.window);
-    let p = new Promise((resolve, reject) =>
-      wait_for_view_load(this.window, resolve)
-    );
+    EventUtils.synthesizeMouseAtCenter(categoryButton, {}, this.window);
 
-    return log_callback(p, aCallback);
+    // Use wait_for_view_load until all open_manager calls are gone.
+    return wait_for_view_load(this.window);
   },
 
-  openType(aCategoryType, aCallback) {
-    return this.open(this.get(aCategoryType), aCallback);
+  openType(categoryType) {
+    return this.open(this.get(categoryType));
   },
 };
 
@@ -754,6 +606,7 @@ function addCertOverride(host, bits) {
           cos.rememberValidityOverride(
             host,
             -1,
+            {},
             securityInfo.serverCert,
             bits,
             false
@@ -792,26 +645,10 @@ function addCertOverrides() {
 
 /** *** Mock Provider *****/
 
-function MockProvider(aUseAsyncCallbacks, aTypes) {
+function MockProvider() {
   this.addons = [];
   this.installs = [];
-  this.callbackTimers = [];
-  this.timerLocations = new Map();
-  this.useAsyncCallbacks =
-    aUseAsyncCallbacks === undefined ? true : aUseAsyncCallbacks;
-  this.types =
-    aTypes === undefined
-      ? [
-          {
-            id: "extension",
-            name: "Extensions",
-            uiPriority: 4000,
-            flags:
-              AddonManager.TYPE_UI_VIEW_LIST |
-              AddonManager.TYPE_SUPPORTS_UNDO_RESTARTLESS_UNINSTALL,
-          },
-        ]
-      : aTypes;
+  this.addonTypes = ["extension"];
 
   var self = this;
   registerCleanupFunction(function() {
@@ -826,12 +663,24 @@ function MockProvider(aUseAsyncCallbacks, aTypes) {
 MockProvider.prototype = {
   addons: null,
   installs: null,
+  addonTypes: null,
   started: null,
-  apiDelay: 10,
-  callbackTimers: null,
-  timerLocations: null,
-  useAsyncCallbacks: null,
-  types: null,
+  queryDelayPromise: Promise.resolve(),
+
+  blockQueryResponses() {
+    this.queryDelayPromise = new Promise(resolve => {
+      this._unblockQueries = resolve;
+    });
+  },
+
+  unblockQueryResponses() {
+    if (this._unblockQueries) {
+      this._unblockQueries();
+      this._unblockQueries = null;
+    } else {
+      throw new Error("Queries are not blocked");
+    }
+  },
 
   /** *** Utility functions *****/
 
@@ -840,7 +689,12 @@ MockProvider.prototype = {
    */
   register: function MP_register() {
     info("Registering mock add-on provider");
-    AddonManagerPrivate.registerProvider(this, this.types);
+    // addonTypes is supposedly the full set of types supported by the provider.
+    // The current list is not complete (there are tests that mock add-on types
+    // other than "extension"), but it doesn't affect tests since addonTypes is
+    // mainly used to determine whether any of the AddonManager's providers
+    // support a type, and XPIProvider already defines the types of interest.
+    AddonManagerPrivate.registerProvider(this, this.addonTypes);
   },
 
   /**
@@ -860,7 +714,7 @@ MockProvider.prototype = {
    */
   addAddon: function MP_addAddon(aAddon) {
     var oldAddons = this.addons.filter(aOldAddon => aOldAddon.id == aAddon.id);
-    var oldAddon = oldAddons.length > 0 ? oldAddons[0] : null;
+    var oldAddon = oldAddons.length ? oldAddons[0] : null;
 
     this.addons = this.addons.filter(aOldAddon => aOldAddon.id != aAddon.id);
 
@@ -1029,26 +883,6 @@ MockProvider.prototype = {
    * Called when the provider should shutdown.
    */
   shutdown: function MP_shutdown() {
-    if (this.callbackTimers.length) {
-      info(
-        "MockProvider: pending callbacks at shutdown(): calling immediately"
-      );
-    }
-    while (this.callbackTimers.length > 0) {
-      // When we notify the callback timer, it removes itself from our array
-      let timer = this.callbackTimers[0];
-      try {
-        let setAt = this.timerLocations.get(timer);
-        info("Notifying timer set at " + (setAt || "unknown location"));
-        timer.callback.notify(timer);
-        timer.cancel();
-      } catch (e) {
-        info("Timer notify failed: " + e);
-      }
-    }
-    this.callbackTimers = [];
-    this.timerLocations = null;
-
     this.started = false;
   },
 
@@ -1059,6 +893,8 @@ MockProvider.prototype = {
    *         The ID of the add-on to retrieve
    */
   async getAddonByID(aId) {
+    await this.queryDelayPromise;
+
     for (let addon of this.addons) {
       if (addon.id == aId) {
         return addon;
@@ -1075,8 +911,10 @@ MockProvider.prototype = {
    *         An array of types to fetch. Can be null to get all types.
    */
   async getAddonsByTypes(aTypes) {
+    await this.queryDelayPromise;
+
     var addons = this.addons.filter(function(aAddon) {
-      if (aTypes && aTypes.length > 0 && !aTypes.includes(aAddon.type)) {
+      if (aTypes && !!aTypes.length && !aTypes.includes(aAddon.type)) {
         return false;
       }
       return true;
@@ -1091,13 +929,15 @@ MockProvider.prototype = {
    *         An array of types or null to get all types
    */
   async getInstallsByTypes(aTypes) {
+    await this.queryDelayPromise;
+
     var installs = this.installs.filter(function(aInstall) {
       // Appear to have actually removed cancelled installs from the provider
       if (aInstall.state == AddonManager.STATE_CANCELLED) {
         return false;
       }
 
-      if (aTypes && aTypes.length > 0 && !aTypes.includes(aInstall.type)) {
+      if (aTypes && !!aTypes.length && !aTypes.includes(aInstall.type)) {
         return false;
       }
 
@@ -1239,7 +1079,7 @@ MockAddon.prototype = {
 
   set appDisabled(val) {
     if (val == this._appDisabled) {
-      return val;
+      return;
     }
 
     AddonManagerPrivate.callAddonListeners("onPropertyChanged", this, [
@@ -1250,8 +1090,6 @@ MockAddon.prototype = {
     this._appDisabled = val;
     var newActive = this.shouldBeActive;
     this._updateActiveState(currentActive, newActive);
-
-    return val;
   },
 
   get userDisabled() {
@@ -1296,7 +1134,7 @@ MockAddon.prototype = {
   },
 
   set permissions(val) {
-    return (this._permissions = val);
+    this._permissions = val;
   },
 
   get applyBackgroundUpdates() {
@@ -1470,7 +1308,7 @@ MockInstall.prototype = {
 
         this.state = AddonManager.STATE_DOWNLOADED;
         this.callListeners("onDownloadEnded");
-
+      // fall through
       case AddonManager.STATE_DOWNLOADED:
         this.state = AddonManager.STATE_INSTALLING;
         if (!this.callListeners("onInstallStarted")) {
@@ -1495,7 +1333,7 @@ MockInstall.prototype = {
         this.callListeners("onInstallEnded");
         break;
       case AddonManager.STATE_DOWNLOADING:
-      case AddonManager.STATE_CHECKING:
+      case AddonManager.STATE_CHECKING_UPDATE:
       case AddonManager.STATE_INSTALLING:
         // Installation is already running
         return;
@@ -1591,29 +1429,9 @@ function waitForCondition(condition, nextTest, errorMsg) {
   };
 }
 
-function getTestPluginTag() {
-  let ph = Cc["@mozilla.org/plugin/host;1"].getService(Ci.nsIPluginHost);
-  let tags = ph.getPluginTags();
-
-  // Find the test plugin
-  for (let i = 0; i < tags.length; i++) {
-    if (tags[i].name == "Test Plug-in") {
-      return tags[i];
-    }
-  }
-  ok(false, "Unable to find plugin");
-  return null;
-}
-
 // Wait for and then acknowledge (by pressing the primary button) the
 // given notification.
 function promiseNotification(id = "addon-webext-permissions") {
-  if (
-    !Services.prefs.getBoolPref("extensions.webextPermissionPrompts", false)
-  ) {
-    return Promise.resolve();
-  }
-
   return new Promise(resolve => {
     function popupshown() {
       let notification = PopupNotifications.getNotification(id);
@@ -1719,69 +1537,73 @@ function assertAboutAddonsTelemetryEvents(events, filters = {}) {
       filters.methods
         ? filters.methods.includes(actual)
         : ABOUT_ADDONS_METHODS.has(actual),
-    object: "aboutAddons",
+    object: actual =>
+      filters.objects
+        ? filters.objects.includes(actual)
+        : actual === "aboutAddons",
   });
 }
 
-function assertTelemetryMatches(events, { filterMethods } = {}) {
-  let snapshot = Services.telemetry.snapshotEvents(
-    Ci.nsITelemetry.DATASET_PRERELEASE_CHANNELS,
-    true
-  );
-
-  if (events.length == 0) {
-    ok(
-      !snapshot.parent || snapshot.parent.length == 0,
-      "There are no telemetry events"
-    );
-    return;
+/* HTML view helpers */
+async function loadInitialView(type, opts) {
+  if (type) {
+    // Force the first page load to be the view we want.
+    let viewId;
+    if (type.startsWith("addons://")) {
+      viewId = type;
+    } else {
+      viewId =
+        type == "discover" ? "addons://discover/" : `addons://list/${type}`;
+    }
+    Services.prefs.setCharPref(PREF_UI_LASTCATEGORY, viewId);
   }
 
-  // Make sure we got some data.
-  ok(
-    snapshot.parent && snapshot.parent.length > 0,
-    "Got parent telemetry events in the snapshot"
-  );
+  let loadCallback;
+  let loadCallbackDone = Promise.resolve();
 
-  // Only look at the related events after stripping the timestamp and category (and optionally filter
-  // out the events related to methods that we are not interested in).
-  let relatedEvents = snapshot.parent
-    .filter(([timestamp, category, method]) => {
-      return (
-        category == "addonsManager" &&
-        (filterMethods ? filterMethods.includes(method) : true)
-      );
-    })
-    .map(relatedEvent => relatedEvent.slice(2, 6));
+  if (opts && opts.loadCallback) {
+    loadCallback = win => {
+      loadCallbackDone = (async () => {
+        // Wait for the test code to finish running before proceeding.
+        await opts.loadCallback(win);
+      })();
+    };
+  }
 
-  // Events are now [method, object, value, extra] as expected.
-  Assert.deepEqual(relatedEvents, events, "The events are recorded correctly");
-}
+  let win = await open_manager(null, null, loadCallback);
+  if (!opts || !opts.withAnimations) {
+    win.document.body.setAttribute("skip-animations", "");
+  }
 
-/* HTML view helpers */
-async function loadInitialView(type) {
-  // Force the first page load to be the view we want.
-  let viewId = type == "discover" ? "discover/" : `list/${type}`;
-  Services.prefs.setCharPref(PREF_UI_LASTCATEGORY, `addons://${viewId}`);
+  // Let any load callback code to run before the rest of the test continues.
+  await loadCallbackDone;
 
-  let managerWindow = await open_manager(null);
-
-  let browser = managerWindow.document.getElementById("html-view-browser");
-  let win = browser.contentWindow;
-  win.managerWindow = managerWindow;
   return win;
 }
 
+function getSection(doc, type) {
+  const SECTION_INDEXES = {
+    enabled: 0,
+    disabled: 1,
+    colorway: 2,
+  };
+  return doc.querySelector(`section[section="${SECTION_INDEXES[type]}"]`);
+}
+
 function waitForViewLoad(win) {
-  return wait_for_view_load(win.managerWindow, undefined, true);
+  return wait_for_view_load(win, undefined, true);
 }
 
 function closeView(win) {
-  return close_manager(win.managerWindow);
+  return close_manager(win);
 }
 
 function switchView(win, type) {
-  return new CategoryUtilities(win.managerWindow).openType(type);
+  return new CategoryUtilities(win).openType(type);
+}
+
+function isCategoryVisible(win, type) {
+  return new CategoryUtilities(win).isTypeVisible(type);
 }
 
 function mockPromptService() {
@@ -1789,7 +1611,7 @@ function mockPromptService() {
   let promptService = {
     // The prompt returns 1 for cancelled and 0 for accepted.
     _response: 1,
-    QueryInterface: ChromeUtils.generateQI([Ci.nsIPromptService]),
+    QueryInterface: ChromeUtils.generateQI(["nsIPromptService"]),
     confirmEx: () => promptService._response,
   };
   Services.prompt = promptService;
@@ -1849,3 +1671,68 @@ async function testUndoPendingUninstall(addonList, addon) {
     "The addon pending uninstall cancelled"
   );
 }
+
+function loadTestSubscript(filePath) {
+  Services.scriptloader.loadSubScript(new URL(filePath, gTestPath).href, this);
+}
+
+function cleanupPendingNotifications() {
+  const { ExtensionsUI } = ChromeUtils.import(
+    "resource:///modules/ExtensionsUI.jsm"
+  );
+  info("Cleanup any pending notification before exiting the test");
+  const keys = ChromeUtils.nondeterministicGetWeakSetKeys(
+    ExtensionsUI.pendingNotifications
+  );
+  if (keys) {
+    keys.forEach(key => ExtensionsUI.pendingNotifications.delete(key));
+  }
+}
+
+function promisePermissionPrompt(addonId) {
+  return BrowserUtils.promiseObserved(
+    "webextension-permission-prompt",
+    subject => {
+      const { info } = subject.wrappedJSObject || {};
+      return !addonId || (info.addon && info.addon.id === addonId);
+    }
+  ).then(({ subject }) => {
+    return subject.wrappedJSObject.info;
+  });
+}
+
+async function handlePermissionPrompt({
+  addonId,
+  reject = false,
+  assertIcon = true,
+} = {}) {
+  const info = await promisePermissionPrompt(addonId);
+  // Assert that info.addon and info.icon are defined as expected.
+  is(
+    info.addon && info.addon.id,
+    addonId,
+    "Got the AddonWrapper in the permission prompt info"
+  );
+
+  if (assertIcon) {
+    ok(info.icon != null, "Got an addon icon in the permission prompt info");
+  }
+
+  if (reject) {
+    info.reject();
+  } else {
+    info.resolve();
+  }
+}
+
+async function switchToDetailView({ id, win }) {
+  let card = getAddonCard(win, id);
+  ok(card, `Addon card found for ${id}`);
+  ok(!card.querySelector("addon-details"), "The card doesn't have details");
+  let loaded = waitForViewLoad(win);
+  EventUtils.synthesizeMouseAtCenter(card, { clickCount: 1 }, win);
+  await loaded;
+  card = getAddonCard(win, id);
+  ok(card.querySelector("addon-details"), "The card does have details");
+  return card;
+}
